import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../utils/ramda-utils';
import {findFirstFieldNameWithFalseValue} from '../utils/transform-utils';
import {externalModelToInternal, transformModelOnToggle} from './form-model-utils';
import {XlReactDipDynamicFormComponent} from './xl-react-dip-dynamic-form-component';
import {findCategoryNameByPropertyName, validateAllFields, validateField} from './xl-react-dip-dynamic-form-service';
import './xl-react-form.less';

export class XlReactDipDynamicForm extends Component {
    static propTypes = {
        allowLookupValues: PropTypes.bool,
        children: PropTypes.oneOfType([
            PropTypes.arrayOf(PropTypes.element),
            PropTypes.element
        ]),
        collapsed: PropTypes.bool,
        context: PropTypes.object,
        dateTimeFormat: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        focusOnFirstInvalidField: PropTypes.bool,
        groupedProperties: PropTypes.array,
        handlers: PropTypes.object,
        mergeContext: PropTypes.func.isRequired,
        message: PropTypes.shape({show: PropTypes.bool}),
        messages: PropTypes.object,
        metadata: PropTypes.object,
        model: PropTypes.object,
        objLabel: PropTypes.string,
        onModelChange: PropTypes.func,
        onPristinityChange: PropTypes.func,
        onValidityChange: PropTypes.func,
        openCategoryGroup: PropTypes.func,
        pristinity: PropTypes.object,
        register: PropTypes.func,
        removeValidationMessages: PropTypes.func,
        toggleCategoryGroup: PropTypes.func,
        updateExternalProperties: PropTypes.func,
        validators: PropTypes.arrayOf(PropTypes.func),
        validity: PropTypes.object
    };

    static defaultProps = {
        allowLookupValues: true,
        collapsed: false,
        context: {},
        dateTimeFormat: 'YYYY-MM-DDTHH:mm:ss.SSSZ',
        disabled: false,
        displayMode: '',
        focusOnFirstInvalidField: true,
        groupedProperties: [],
        handlers: {},
        message: {show: false},
        messages: {},
        metadata: {},
        model: {},
        onModelChange: R.F,
        onPristinityChange: R.F,
        onValidityChange: R.F,
        pristinity: {},
        validators: [],
        validity: {}
    };

    constructor(props) {
        super(props);
        this.componentApis = {};
        RU.invoke('register', [this.api], props);
        RU.invoke('onValidityChange', [props.validity], props);
        RU.invoke('onPristinityChange', [props.pristinity], props);
    }

    componentDidMount() {
        if (this.props.focusOnFirstInvalidField) {
            this.api.focusOnFirstInvalidField();
        }
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        if (this.isNotEqual(this.props, nextProps, ['disabled', 'displayMode', 'message', 'metadata']) ||
            !R.equals(this.props.model['validation-messages'], nextProps.model['validation-messages'])) {
            this.updateValidityForAllFields(nextProps);
        }
    }

    shouldComponentUpdate(nextProps) {
        const propsToReRender = [
            'collapsed',
            'context',
            'disabled',
            'displayMode',
            'groupedProperties',
            'message',
            'messages',
            'metadata',
            'model',
            'objLabel',
            'pristinity',
            'validity'
        ];
        return this.isNotEqual(this.props, nextProps, propsToReRender);
    }

    handleLookupToggle = (ciPropName, isOn, value) => {
        const model = transformModelOnToggle(this.props.model, ciPropName, isOn, this.memorized[ciPropName]);
        this.props.updateExternalProperties(model);
        RU.invoke('onModelChange', [model], this.props);
        this.props.mergeContext({pristinity: {[ciPropName]: false}});
        this.memorized[ciPropName] = value;
    };

    onModelChangeHandlerCreator = (propName) => async (value, shouldUpdatePristinity = true) => {
        const {
            context,
            mergeContext,
            model,
            pristinity,
            removeValidationMessages,
            validity
        } = this.props;

        const externalProperties = model['external-properties'] || {};
        const getPath = () => R.prop(propName, externalProperties) ? ['external-properties', propName] : [propName];
        const getTransformedModel = () => RU.invoke('transformModel', [value], context[propName]) || value;

        const computedContext = await RU.invoke(
            'computeContext',
            [mergeContext, context[propName], getTransformedModel()],
            context[propName]
        ) || {};

        const newContext = {
            ...context[propName],
            ...computedContext
        };
        const pristine = shouldUpdatePristinity ? false : pristinity[propName];
        const {message, valid} = validateField(this.props, propName, newContext, pristine, value);
        mergeContext({
            messages: {[propName]: message},
            model: R.assocPath(getPath(), getTransformedModel(), {}),
            pristinity: {[propName]: pristine},
            validity: {[propName]: valid}
        });

        const callChangeHandler = (newValue, objectWithValues, handler) => {
            if (newValue !== objectWithValues[propName]) {
                RU.invoke(handler, [{...objectWithValues, [propName]: newValue}], this.props);
            }
        };

        RU.invoke('onModelChange', [{...R.assocPath(getPath(), value, model)}], this.props);
        removeValidationMessages(propName);
        callChangeHandler(pristine, pristinity, 'onPristinityChange');
        callChangeHandler(valid, validity, 'onValidityChange');
    };

    updateValidityForAllFields(props) {
        const validation = validateAllFields(props);
        props.mergeContext(validation);
        RU.invoke('onValidityChange', [validation.validity], props);
    }

    isNotEqual = (left, right, names) =>
        !R.reduce((acc, name) => acc && R.equals(left[name], right[name]), true, names);

    api = {
        focusFieldByName: (fieldName) => {
            const isHiddenProperty = R.isNil(this.componentApis[fieldName]);
            if (fieldName && !isHiddenProperty) {
                this.componentApis[fieldName].focus();
            }
        },
        focusOnFirstInvalidField: () => {
            const nameOfInvalidField = findFirstFieldNameWithFalseValue(this.props.validity);
            if (nameOfInvalidField) {
                this.componentApis[nameOfInvalidField].focus();
                const categoryName = findCategoryNameByPropertyName(this.props.metadata, nameOfInvalidField);
                this.props.openCategoryGroup(categoryName);
            }
        },
        resetPristinity: () => {
            R.pipe(
                R.keys,
                R.reduce((acc, key) => R.assoc(key, true, acc), {}),
                (pristinity) => this.props.mergeContext({pristinity})
            )(this.props.pristinity);
        }
    };

    memorized = {};

    render() {
        const {
            allowLookupValues,
            children,
            collapsed,
            context,
            dateTimeFormat,
            disabled,
            displayMode,
            handlers,
            groupedProperties,
            mergeContext,
            message,
            messages,
            metadata,
            model,
            objLabel,
            pristinity,
            toggleCategoryGroup,
            validators,
            validity
        } = this.props;

        const onRegister = (fieldName, api) => {
            this.componentApis[fieldName] = api;
        };

        return <XlReactDipDynamicFormComponent {...({
            allowLookupValues,
            children,
            collapsed,
            context,
            dateTimeFormat,
            disabled,
            displayMode,
            groupedProperties,
            handlers,
            mergeContext,
            message,
            messages,
            metadata,
            model: externalModelToInternal(model),
            objLabel,
            onLookupToggle: this.handleLookupToggle,
            onModelChangeCreator: this.onModelChangeHandlerCreator,
            onRegister,
            pristinity,
            toggleCategoryGroup,
            validators,
            validity
        })}
        />;
    }
}
