import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';

import RU from '../utils/ramda-utils';
import {nextId} from '../utils/id-generator';
import {XlReactTabSystem, skins} from './xl-react-tab-system';

export class XlReactTabSystemContainer extends PureComponent {
    static propTypes = {
        actions: PropTypes.arrayOf(PropTypes.shape({
            action: PropTypes.func.isRequired,
            label: PropTypes.string.isRequired
        })),
        activeTabId: PropTypes.string,
        config: PropTypes.shape({openAnotherTabIfCurrentIsClosed: PropTypes.bool}),
        data: PropTypes.arrayOf(PropTypes.shape({
            fullTitle: PropTypes.string,
            iconUrl: PropTypes.string,
            loading: PropTypes.bool,
            pristine: PropTypes.bool.isRequired,
            tabId: PropTypes.string.isRequired,
            title: PropTypes.string.isRequired,
            viewType: PropTypes.string.isRequired
        })).isRequired,
        loaderElement: PropTypes.element,
        onChangeActiveTabId: PropTypes.func.isRequired,
        onChangeData: PropTypes.func.isRequired,
        onNewTab: PropTypes.func,
        register: PropTypes.func,
        skin: PropTypes.string,
        tabDefinitions: PropTypes.object.isRequired
    };

    static defaultProps = {config: {}};

    UNSAFE_componentWillMount() {
        RU.invoke('register', [this.api], this.props);
    }

    handleCloseTab = (tabId) => {
        this.api.removeTab(tabId);
    };

    handleTabSelected = (tab) => {
        this.api.openTab(tab);
    };

    handleOnNewTab = () => {
        RU.invoke('onNewTab', [], this.props);
    };

    api = {
        addTab: (tabInfo) => {
            if (this.api.getTab(tabInfo.tabId)) {
                return this.api.openTab(tabInfo);
            }

            const newTabInfo = this.createTabInfo(tabInfo);
            this.props.onChangeData([...this.props.data, newTabInfo]);
            this.props.onChangeActiveTabId(newTabInfo.tabId);
            return newTabInfo.tabId;
        },
        changeIconUrl: (tabId, iconUrl) => {
            this.api.updateTabInfo(tabId, {iconUrl});
        },
        changeLoading: (tabId, loading) => {
            this.api.updateTabInfo(tabId, {loading});
        },
        changePristinity: (tabId, pristine) => {
            this.api.updateTabInfo(tabId, {pristine});
        },
        changeTitle: (tabId, fullTitle, title) => {
            this.api.updateTabInfo(tabId, {fullTitle, title});
        },
        getActiveTab: () => {
            const tabId = this.props.activeTabId;
            return tabId ? this.api.getTab(tabId) : undefined;
        },
        getTab: (tabId) => R.find(R.propEq('tabId', tabId), this.props.data),
        openTab: (tab) => {
            this.props.onChangeActiveTabId(tab.tabId);
            return tab.tabId;
        },
        removeAllTabs: () => {
            this.props.onChangeData([]);
            this.props.onChangeActiveTabId(undefined);
        },
        removeTab: (removedTabId) => {
            const {config} = this.props;
            const newData = R.filter(R.pipe(R.propEq('tabId', removedTabId), R.not), this.props.data);
            const newActiveTabId = R.pipe(R.pluck('tabId'), R.head)(newData);
            this.props.onChangeData(newData);

            const isCurrentTabClosed = removedTabId === this.props.activeTabId;
            const openAnotherTab = isCurrentTabClosed && config.openAnotherTabIfCurrentIsClosed !== false;

            if (openAnotherTab) {
                this.api.openTab({tabId: newActiveTabId});
            }
            return newActiveTabId;
        },
        replaceTabContent: (tabId, viewType) => {
            this.api.updateTabInfo(tabId, {viewType});
            return this.api.getTab(tabId);
        },
        updateTabInfo: (tabId, newFields) => {
            const newTabs = R.map((tab) => R.equals(tabId, tab.tabId) ? {...tab, ...newFields} : tab)(this.props.data);
            this.props.onChangeData(newTabs);
        }
    };

    createTabInfo = (tabInfo) => {
        const {
            loading = false,
            tabId = nextId(),
            pristine = true,
            ...rest
        } = tabInfo;

        return {
            ...rest,
            loading,
            pristine,
            tabId
        };
    };

    render() {
        return <XlReactTabSystem
            actions={this.props.actions}
            activeTabId={this.props.activeTabId}
            data={this.props.data}
            loaderElement={this.props.loaderElement}
            onCloseTab={this.handleCloseTab}
            onNewTab={this.handleOnNewTab}
            onTabSelected={this.handleTabSelected}
            skin={this.props.skin || skins.DEFAULT}
            tabDefinitions={this.props.tabDefinitions}
        />;
    }
}
