var os = require('os');

var AsciiTable = require('ascii-table');
var BaseDsl = require('protractor-base-dsl');
var R = require('ramda');
var yargs = require('yargs');
var DEFAULT_TIMEOUT = 600000;
var DEFAULT_EXPECTATION_TIMEOUT = 20000;
var ScreenShotReporter = require('protractor-screenshot-reporter');

var browserName = process.env.SELENIUM_TEST_BROWSER || 'chrome';
var platform = (process.env.SELENIUM_TEST_PLATFORM || 'any');
var platformName = (process.env.SELENIUM_TEST_PLATFORM_NAME || 'any').toLowerCase();
var vr = require('./visual-review').getVr();
var afterLaunch = require('./after-launch').afterLaunch;

var proxyPort = yargs.argv.params.proxyPort;
var seleniumVersions = require('../../../gulp/tasks/selenium/selenium-versions.json');

var hostname = require('../../../gulp/utils/hostname');

global.vr = vr;

var browserWidth = 1024;
var browserHeight = 768;

function printBanner() {
    var table = new AsciiTable('Configuration');
    table
        .addRow('Host', R.propOr('localhost', 'SELENIUM_TEST_ADDR', process.env))
        .addRow('VisualTesting host', R.propOr('Not set (DISABLED)', 'SELENIUM_TEST_VR_HOSTNAME', process.env))
        .addRow('Browser', R.propOr(`Not set. Using default. (${browserName})`, 'SELENIUM_TEST_BROWSER', process.env))
        .addRow('Platform', R.propOr(`Not set. Using local: (${os.platform()})`, 'SELENIUM_TEST_PLATFORM', process.env))
        .addRow('Platform name', R.propOr(`Not set. Using local: (${os.platform()})`, 'SELENIUM_TEST_PLATFORM_NAME', process.env));
    console.log(table.toString());
}

exports.config = {
    allScriptsTimeout: DEFAULT_TIMEOUT,
    baseUrl: 'http://' + hostname + ':' + proxyPort + '/',
    capabilities: {
        browserName,
        platform,
        platformName,
        ...browserName === 'chrome' ? {
            chromeOptions: {
                args: ['--window-size=' + browserWidth + ',' + (browserHeight + 120)]
            }
        } : {},
        'se:ieOptions': {requireWindowFocus: true}
    },
    framework: 'jasmine2',
    troubleshoot: true,
    getPageTimeout: DEFAULT_TIMEOUT,
    jasmineNodeOpts: {
        showColors: true,
        defaultTimeoutInterval: DEFAULT_TIMEOUT,
        print: R.F
    },
    rootElement: 'body',
    seleniumAddress: (process.env.SELENIUM_TEST_ADDR || null),
    suites: {
        'button': '../scenario/button/**/*.js',
        'dip': '../scenario/dip/**/*.js',
        'form': '../scenario/form/**/*.js',
        'tab': '../scenario/tab/**/*.js',
        'widget': '../scenario/widget/**/*.js'
    },
    onPrepare: function () {
        require('@babel/register')({
            presets: [
                '@babel/preset-env'
            ]
        });

        addLibraries({
            json: 'format-json',
            KeyCodes: 'keycode-js',
            moment: 'moment',
            requestify: 'requestify',
            requestPromise: 'request-promise',
            Q: 'q',
            R: 'ramda',
            ReactTestUtils: 'react-dom/test-utils'
        });
        global.vr = vr;
        global.appHost = hostname;
        global.defaultExpectationTimeout = DEFAULT_EXPECTATION_TIMEOUT;
        global.defaultBrowserWidth = browserWidth;
        global.defaultBrowserHeight = browserHeight;
        global.genericDateFormat = 'MMM D, YYYY';

        global.Action = BaseDsl.Action;
        global.ActionUtil = BaseDsl.ActionUtil;
        global.Condition = BaseDsl.Condition;
        global.Expectation = BaseDsl.Expectation;
        global.ElementUtil = BaseDsl.ElementUtil;
        global.WaitCondition = BaseDsl.WaitCondition;
        global.Window = BaseDsl.Window;
        global.XPath = BaseDsl.XPath;

        var reporters = require('jasmine-reporters');
        var Jasmine2HtmlReporter = require('protractor-jasmine2-html-reporter');
        var SpecReporter = require('jasmine-spec-reporter').SpecReporter;

        jasmine.getEnv().addReporter(new reporters.JUnitXmlReporter({
            consolidateAll: false,
            savePath: 'build/test-results/protractor'
        }));

        jasmine.getEnv().addReporter(new ScreenShotReporter({
            baseDirectory: 'build/screenshots'
        }));

        jasmine.getEnv().addReporter(new Jasmine2HtmlReporter({
            savePath: 'build/reports/e2e/',
            takeScreenshots: true,
            takeScreenshotsOnlyOnFailures: true
        }));

        jasmine.getEnv().addReporter(new SpecReporter({
            suite: {
                displayNumber: true
            },
            spec: {
                displayErrorMessages: true,
                displayStacktrace: true,
                displaySuccessful: true,
                displayFailed: true,
                displayPending: true,
                displayDuration: true
            }
        }));

        var failFast = require('jasmine-fail-fast');
        jasmine.getEnv().addReporter(failFast.init());

        rmdir = require('rimraf');
        rmdir('build/reports', function (error) {
        });

        function addComponentFolder(folderName) {
            var files = require('glob').sync('../dsl/component/' + folderName + '/**/*.js', {cwd: __dirname});
            R.forEach(require, files);
        }

        var dslCommonFiles = require('glob').sync('../dsl/common/**/*.js', {cwd: __dirname});
        console.log('dslCommonFiles', dslCommonFiles);
        R.forEach(require, dslCommonFiles);

        addComponentFolder('base');
        addComponentFolder('widget');
        addComponentFolder('dip');
        addComponentFolder('form');
        addComponentFolder('tab');

        return browser.waitForAngularEnabled(false)
            .then(function() {
                if (browserName === 'firefox') {
                    return browser.manage().window().setSize(global.defaultBrowserWidth, global.defaultBrowserHeight);
                }

                if (browserName === 'internet explorer') {
                    return browser.executeScript(function () {
                        window.resizeTo(1024, 768);
                    });
                }
            }).then(function() {
                return Navigation.openHomePage();
            }).then(function() {
                return browser.manage().timeouts().setScriptTimeout(DEFAULT_TIMEOUT);
            });

        function addLibraries(libraries) {
            for (var key in libraries) {
                global[key] = require(libraries[key]);
            }
        }
    },

    onCleanUp: function (exitCode) {
        var q = require('q');
        var del = require('del');
        var deferred = q.defer();

        var reportsFolder = 'build/reports';
        var reportsFile = reportsFolder + '/consoleErrors.txt';
        var fs = require('fs');
        var archiver = require('archiver');

        fs.exists(reportsFile, function (exists) {
            if (!exists) {
                deferred.resolve(exitCode);
                return;
            }

            var archive = archiver.create('zip', {});

            var output = fs.createWriteStream(reportsFolder + '/consoleErrors.zip');
            archive.pipe(output);

            output.on('close', function () {
                del([reportsFolder + '/*.txt']).then(function () {
                    deferred.resolve(exitCode);
                });
            });

            archive.file(reportsFolder + '/consoleErrors.txt', {name: 'consoleErrors.txt'});
            archive.finalize();
        });

        return deferred.promise;
    },
    beforeLaunch: function () {
        printBanner();
        return vr.initRun('xld-ci-explorer', platformName + ' ' + browserName + ' ' + yargs.argv.suite);
    },
    afterLaunch: function (exitCode) {
        return afterLaunch(exitCode, vr)
    },
    params: {
        propertiesFn: function () {
            return {
                browser: browserName,
                os: platformName
            };
        },
        visualreview: vr
    },
    jvmArgs: ['-Dwebdriver.gecko.driver=./node_modules/webdriver-manager/selenium/geckodriver-' + seleniumVersions.gecko],
    debug: true
};
