import R from 'ramda';

import invalidCommonCategoryModel from './invalid-common-category-model.json';
import allInvalidCategoriesModel from './all-invalid-categories-model.json';
import noInvalidCategoriesModel from './no-invalid-categories-model.json';
import {
    getInitialContext,
    getInvalidCategoryNames,
    groupByCategory
} from '~/app/xl-react-form/xl-react-dip-dynamic-form-service';
import {getInvalidDateValidators} from '~/app/xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import {
    filterItems,
    invalidCiValidator,
    prepareViewModel,
    sortItems
} from '~/app/xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service';

jest.mock('../../../app/xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service');
jest.mock('../../../app/xl-react-widget/xl-react-widget-date/xl-react-widget-date');

describe('XlReactDipDynamicFormServiceTest', () => {
    it('should sort by category name insensitive to upper case, but having Common in precedence as an exception ' +
        'case', () => {
        const result = groupByCategory([
            {category: 'water'},
            {category: 'Common'},
            {category: 'Common'},
            {category: 'Milk'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'Milk', 'water']);
    });

    it('should default property categories to "Common" if they are not defined', () => {
        const result = groupByCategory([
            {name: 'A'},
            {name: 'B'},
            {category: 'Other', name: 'C'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'Other']);
        expect(R.find(R.propEq('category', 'Common'), result).properties).toEqual([
            {category: 'Common', name: 'A'},
            {category: 'Common', name: 'B'}
        ]);
    });

    it('should group properties by category', () => {
        const result = groupByCategory([
            {category: 'One', name: 'A'},
            {name: 'B'},
            {category: 'Three', name: 'C'},
            {category: 'One', name: 'D'},
            {category: 'Common', name: 'E'}
        ]);
        expect(R.map(R.prop('category'), result)).toEqual(['Common', 'One', 'Three']);
        expect(R.find(R.propEq('category', 'Common'), result).properties).toEqual([
            {category: 'Common', name: 'B'},
            {category: 'Common', name: 'E'}
        ]);
        expect(R.find(R.propEq('category', 'One'), result).properties).toEqual([
            {category: 'One', name: 'A'},
            {category: 'One', name: 'D'}
        ]);
        expect(R.find(R.propEq('category', 'Three'), result).properties).toEqual([
            {category: 'Three', name: 'C'}
        ]);
    });

    it('should set category options', () => {
        const result = groupByCategory([{name: 'A'}], undefined, undefined, {Common: {titleHidden: true}});
        expect(R.map(R.prop('category'), result)).toEqual(['Common']);
        expect(R.find(R.propEq('category', 'Common'), result).categoryOptions).toEqual({titleHidden: true});
    });

    describe('open property', () => {
        it('should set "open" property to "true" if collapsed parameter is null', () => {
            const result = groupByCategory([{category: 'Common'}], null);
            expect(R.nth(0, result).isOpen).toBeTruthy();
        });

        it('should set "open" property to "true" if collapsed parameter is undefined', () => {
            const result = groupByCategory([{category: 'Common'}], undefined);
            expect(R.nth(0, result).isOpen).toBeTruthy();
        });

        it('should set "open" property to what is specified in collapsed parameter', () => {
            const result = groupByCategory([{category: 'Common'}], false);
            expect(R.nth(0, result).isOpen).toBeFalsy();
        });

        it('should set "open" property to "false" when collapse is "auto" and no VISIBLE REQUIRED fields', () => {
            const result = groupByCategory([
                {category: 'Common', name: 'A'},
                {category: 'Common', name: 'B'}
            ], 'auto');
            expect(R.nth(0, result).isOpen).toBeFalsy();
        });

        const autoCollapseTestPipe = R.pipe(
            R.curry(groupByCategory)(R.__, 'auto', undefined),
            R.nth(0),
            R.prop('isOpen')
        );

        it('should set "open" property to "true" when collapse is "auto" and there are REQUIRED fields', () => {
            expect(autoCollapseTestPipe([
                {category: 'Common', name: 'A', required: true},
                {category: 'Common', name: 'B'}
            ])).toBeTruthy();

            expect(autoCollapseTestPipe([
                {category: 'Common', hidden: true, name: 'A', required: true},
                {category: 'Common', name: 'B', required: true}
            ])).toBeTruthy();
        });

        it('should set "open" property to "false" when collapse is "auto" and there are REQUIRED fields but they ' +
            'all are hidden', () => {
            expect(autoCollapseTestPipe([
                {category: 'Common', hidden: true, name: 'A', required: true},
                {category: 'Common', hidden: true, name: 'B', required: true}
            ])).toBeFalsy();
        });

        it('should set "open" property from old group properties', () => {
            const result = groupByCategory([
                {category: 'Common', name: 'A'},
                {category: 'Uncommon', name: 'B'}
            ], undefined, [
                {category: 'Common', isOpen: false, name: 'A'},
                {category: 'Uncommon', isOpen: true, name: 'B'}
            ]);
            expect(R.nth(0, result).isOpen).toBeFalsy();
            expect(R.nth(1, result).isOpen).toBeTruthy();
        });
    });

    describe('getInvalidCategoryNames', () => {
        it('should return the array of names of categories where at least one property is invalid', () => {
            expect(getInvalidCategoryNames(invalidCommonCategoryModel)).toEqual(['Common']);
            expect(getInvalidCategoryNames(allInvalidCategoriesModel)).toEqual(['Common', 'Personal Data']);
            expect(getInvalidCategoryNames(noInvalidCategoriesModel)).toEqual([]);
        });
    });

    describe('getInitialContext', () => {
        it('should return default initial state for form with ' +
            'boolean, ci, enum, integer, mapStringString, setOString or string  field', () => {
            const expectDefaultInitialStateForField = (name, kind, val) => {
                const model = {[name]: false};
                const metadata = {properties: [{kind, name}]};
                const result = getInitialContext({metadata}, model);
                expect(R.keys(result)).toStrictEqual([name]);
                expect(R.keys(result[name])).toStrictEqual(['computeContext', 'getValidators', 'transformModel']);
                expect(result[name].computeContext()).toStrictEqual({});
                expect(result[name].getValidators()).toStrictEqual([]);
                expect(result[name].transformModel(val)).toBe(val);
            };

            expectDefaultInitialStateForField('bool', 'BOOLEAN', false);
            expectDefaultInitialStateForField('ci', 'CI', {id: 'id'});
            expectDefaultInitialStateForField('enum', 'ENUM', 'val');
            expectDefaultInitialStateForField('mapStringString', 'MAP_STRING_STRING', {key: 'value'});
            expectDefaultInitialStateForField('setOString', 'SET_OF_STRING', ['string']);
            expectDefaultInitialStateForField('string', 'STRING', 'str');
        });

        it('should return initial state for form with date field', () => {
            const validators = [() => false];
            getInvalidDateValidators.mockImplementation(R.always(validators));
            const model = {date: false};
            const metadata = {properties: [{kind: 'DATE', name: 'date'}]};
            const result = getInitialContext({metadata}, model);
            expect(R.keys(result)).toStrictEqual(['date']);
            expect(R.keys(result.date)).toStrictEqual(['computeContext', 'getValidators', 'transformModel']);
            expect(result.date.computeContext()).toStrictEqual({});
            expect(result.date.getValidators()).toStrictEqual(validators);
            const date = new Date();
            expect(result.date.transformModel(date)).toBe(date);
        });

        it('should return initial state for form with listOfString field', () => {
            const model = {list: false};
            const metadata = {properties: [{kind: 'LIST_OF_STRING', name: 'list'}]};
            const result = getInitialContext({metadata}, model);
            expect(R.keys(result)).toStrictEqual(['list']);
            expect(R.keys(result.list)).toStrictEqual(['computeContext', 'getValidators', 'transformModel', 'typed']);
            expect(result.list.computeContext()).toStrictEqual({});
            expect(result.list.getValidators()).toStrictEqual([]);
            const list = ['str'];
            expect(result.list.transformModel(list)).toBe(list);
            expect(result.list.typed).toBe('');
        });

        const validator = () => false;
        filterItems.mockImplementation(R.always(['a']));
        invalidCiValidator.mockImplementation(R.always(validator));
        prepareViewModel.mockImplementation(R.identity);
        const unsorted = ['c', 'a', 'b'];
        const sorted = ['a', 'b', 'c'];
        const validated = [{model: 'a', valid: true}];
        const model = {list: unsorted};
        sortItems.mockImplementation(R.always(sorted));
        const validateItems = jest.fn().mockReturnValue(validated);

        async function expectCollectionOfCi(result, initialValidatedItems, validatedItems, transformedModel) {
            expect(R.keys(result)).toStrictEqual(['list']);
            expect(R.keys(result.list)).toStrictEqual([
                'computeContext', 'getValidators', 'transformModel', 'searchTerm', 'validatedItems', 'visibleItems'
            ]);
            expect(result.list.getValidators()).toStrictEqual([validator]);
            expect(result.list.transformModel(model.list)).toBe(transformedModel);
            expect(result.list.searchTerm).toBe('');
            expect(result.list.validatedItems).toStrictEqual(initialValidatedItems);
            expect(result.list.visibleItems).toStrictEqual(initialValidatedItems);
            const mergeContext = jest.fn();
            const context = await result.list.computeContext(mergeContext, {}, validatedItems);
            expect(mergeContext).toHaveBeenCalledTimes(2);
            expect(mergeContext).toHaveBeenNthCalledWith(1, {context: {list: {loadingItems: true}}});
            const newContext = {
                loadingItems: false,
                validatedItems,
                visibleItems: ['a']
            };
            expect(mergeContext).toHaveBeenNthCalledWith(2, {context: {list: newContext}});
            expect(context).toStrictEqual(newContext);
        }

        it('should return initial state for form with listOfCi field', async () => {
            const metadata = {properties: [{kind: 'LIST_OF_CI', name: 'list'}]};
            const result = getInitialContext({metadata}, model);
            await expectCollectionOfCi(result, unsorted, unsorted, model.list);
        });

        it('should return initial state for form with setOfCi field', async () => {
            const metadata = {properties: [{kind: 'SET_OF_CI', name: 'list'}]};
            const handlers = {SET_OF_CI: {validateItems}};
            const result = getInitialContext({handlers, metadata}, model);
            await expectCollectionOfCi(result, sorted, validated, sorted);
            expect(validateItems).toHaveBeenCalledWith(['a']);
        });
    });
});
