import React from 'react';
import {mount} from 'enzyme';
import moment from 'moment';
import timekeeper from 'timekeeper';
import {XlReactWidgetDate} from '~/app/xl-react-widget/xl-react-widget-date/xl-react-widget-date';

const now = new Date();
timekeeper.freeze(now);

describe('XlReactWidgetDate', () => {
    const onModelChange = jest.fn();

    const defaultProps = {
        metadata: {label: 'Date', placeholder: 'placeholder'},
        onModelChange
    };
    let wrapper;

    beforeEach(() => {
        wrapper = mount(<XlReactWidgetDate {...defaultProps}/>);
    });

    it('should render correctly', () => {
        const datePicker = wrapper.find('.xl-react-components.xl-react-widget-date');
        expect(wrapper.find('XlReactWidgetLabel')).toBeDefined();
        expect(wrapper.find('.xl-react-widget-date')).toBeDefined();
        expect(datePicker).not.toBeDisabled();
        expect(wrapper).toHaveState('pristine', true);
    });

    it('should render a datepicker', () => {
        expect(wrapper.find('.xl-react-components.xl-react-widget-date')).toBeDefined();
    });

    it('should show the placeholder if none is passed', () => {
        const datePicker = wrapper.find('.xl-react-components.xl-react-widget-date');
        expect(onModelChange).toBeCalledWith(now, true);
        expect(datePicker.find('.react-datepicker__input-container input')).toHaveProp('placeholder', 'placeholder');
    });

    it('should disable the datepicker when property is disabled', () => {
        wrapper.setProps({disabled: true});
        const datePicker = wrapper.find('.xl-react-components.xl-react-widget-date');
        const input = datePicker.find('.react-datepicker__input-container input');
        expect(input).toHaveProp('disabled', true);
    });

    it('should be able to update the datePicker through props', () => {
        const date = moment('1989-12-12 0:00', 'YYYY-MM-DD HH:mm');
        wrapper.setProps({model: date});
        const input = wrapper.find('.react-datepicker__input-container input');
        expect(input).toHaveProp('value', 'Dec 12, 1989');
    });

    it('should show the date passed as prop', () => {
        const date = moment('2017-01-01 0:00', 'YYYY-MM-DD HH:mm');
        const wrapper2 = mount(<XlReactWidgetDate {...defaultProps} model={date}/>);
        expect(moment(wrapper2.state('model')).isSame(date)).toBe(true);
    });

    it('should accept different date foramts', () => {
        const date = moment('2017-01-04 0:00', 'YYYY-MM-DD HH:mm');
        const props = {...defaultProps, dateFormat: 'YYYY/MM/DD'};
        const wrapper2 = mount(<XlReactWidgetDate {...props} model={date}/>);
        const input = wrapper2.find('.react-datepicker__input-container input');
        expect(input).toHaveProp('value', '2017/01/04');
    });

    it('should be able to call focus and reset pristinity', () => {
        const date = moment('2017-01-01 0:00', 'YYYY-MM-DD HH:mm');
        let registeredApi;
        const registerComponent = (api) => {
            registeredApi = api;
        };
        mount(<XlReactWidgetDate {...defaultProps} model={date} register={registerComponent}/>);
        registeredApi.focus();
        registeredApi.resetPristinity();
    });
});

